/*
 * Mainboart_UART_functions.c
 *
 * Created: 21.01.2014 10:18:41
 *  Author: Alex
 */ 

#include "Mainboard_UART_functions.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "uart.h"


void process_rxbuf (){
	
}

void send_ADC_raw(uint32_t adcreceive)
{
	uint8_t sendbuf;
	uint8_t ascii_convert [16] = {A_0,A_1,A_2,A_3,A_4,A_5,A_6,A_7,A_8,A_9,A,B,C,D,E,F};
	uint8_t ascii_buf=0;
	
	uart_puts_P("RAW: ");	
	
	ascii_buf = ((adcreceive>>20) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_puts(sendbuf);
	ascii_buf = ((adcreceive>>16) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((adcreceive>>12) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((adcreceive>>8) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((adcreceive>>4) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = (adcreceive & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);

	// Send Carriage Return + Line Feed
	uart_puts("\r");	
	uart_puts("\n");	
	
	};

void send_ADC_temp (uint16_t temp)
{
	// Send ASCII TimeBlock: [T|E|M|P|:|TEMPVAL(Hex,4ASCII)|CR|LF]
		uint8_t sendbuf;
		uint8_t ascii_convert [16] = {A_0,A_1,A_2,A_3,A_4,A_5,A_6,A_7,A_8,A_9,A,B,C,D,E,F};
		uint8_t ascii_buf=0;
			
		// Send Temp_Measurement
		uart_puts_P("TEMP: ");	
	
		// Send Temperature, Ascii converted
		ascii_buf = ((temp>>12) & 0x0F);
		sendbuf = ascii_convert[ascii_buf];
		uart_putc(sendbuf);
		ascii_buf = ((temp>>8) & 0x0F);
		sendbuf = ascii_convert[ascii_buf];
		uart_putc(sendbuf);
		ascii_buf = ((temp>>4) & 0x0F);
		sendbuf = ascii_convert[ascii_buf];
		uart_putc(sendbuf);
		ascii_buf = (temp & 0x0F);
		sendbuf = ascii_convert[ascii_buf];
		uart_putc(sendbuf);
		
		// Send Carriage Return + Line-Feed
		uart_puts("\r");
		uart_puts("\n");
}

void send_ADC_data (uint8_t active_module, uint8_t active_channel, bool Speedmode, uint16_t adc_value, uint16_t timestamp) {
	
	// Send ASCII DataBlock: [M|#|;|C|#|;|S|0/1|;|ADCVAL(Hex,4ASCII)|;| |TIMERVAL(Hex,4ASCII)|CR|LF]
	
	uint8_t ascii_convert [16] = {A_0,A_1,A_2,A_3,A_4,A_5,A_6,A_7,A_8,A_9,A,B,C,D,E,F};
	uint8_t ascii_buf=0;
	uint8_t sendbuf;
	
	
	uart_puts("M");				// Send Active Module
	switch (active_module)		// Acive Module Number
	{
		case 0:{
			uart_puts("0");
			break;
		}
		case 1:{
			uart_puts("1");
			break;
		}
		case 2:{
			uart_puts("2");	
			break;
		}
		case 3:{
			uart_puts("3");
			break;
		}
	}
	uart_puts(";");				// Send Semicolon
	uart_puts("C");				// Send Active Channel					
	switch (active_channel)		// Active Channel Number
	{
		case 0:{
			uart_puts("0");
			break;
		}
		case 1:{
			uart_puts("2");
			break;
		}
		case 2:{
			uart_puts("3");
			break;
		}
		case 3:{
			uart_puts("4");
			break;
		}
	}
	
	// Send Semicolon
	uart_puts(";");
	// Send Speedmode True/False
	uart_puts("S");			
	if (Speedmode) uart_puts("1");		// ASCII Speedmode Bool "1"
	else uart_puts("0");				// ASCII Speedmode Bool "0"
	uart_puts(";");						// Send Semicolon					
			
	// Send ADC-Value ASCII Converted
	ascii_buf = ((adc_value>>12) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((adc_value>>8) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((adc_value>>4) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = (adc_value & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	
	// Send Semicolon
	uart_puts(";");
	
	// Send timer-Value ASCII Converted
	ascii_buf = ((timestamp>>12) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((timestamp>>8) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = ((timestamp>>4) & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	ascii_buf = (timestamp & 0x0F);
	sendbuf = ascii_convert[ascii_buf];
	uart_putc(sendbuf);
	
	// Send Carriage Return + Linefeed
	uart_puts("\r");
	uart_puts("\n");		
}

void Timer_init(uint16_t ctcval) {
	ctcval = 195;					// Output Compare Match with prescaler 1/1024: Every ~10ms
	PRR &= ~(1<<PRTIM1);			// Power Reduction Register: Enable Timer/Counter1
	OCR1A = ctcval;
	TCCR1B |= (1<<CS12)|(1<<CS10)|(1<<WGM12);	// Prescaler 1/1024, Clear Timer on Compare match (CTC), Mode 4
	TIMSK1 |= (1<<OCIE1A);									// Timer/Counter1 Output Compare Interrupt Request
}

void Timer_reset() {
	uint16_t val = 0;
	TCNT1 = val;
}

void Timer_disable_interrupt()
{
	TIMSK1 &= ~(1<<OCIE1A);	
}

void Timer_enable_interrupt()
{
	TIMSK1 |= (1<<OCIE1A);
}